﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Net;
using System.Diagnostics;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Content;

namespace CS_3505_Project_06
{
    public class lobbyGUI : ILobby
    {
        float scale;

        Texture2D background;
        Texture2D spotLight;
        Texture2D cs;
        Texture2D selectGameScreen;

        Vector2 backgroundPos;
        Vector2 spotLightPos;
        Vector2 spotLightCenter;
        Vector2 csPos;

        Vector2 zero;
        Vector2 spotLightVelocity;

        int MaxX;
        int MinX;
        int MaxY;
        int MinY;

        Texture2D checkedBox;
        Texture2D deselectBox;
        Texture2D emptySelectBox;
        Texture2D menuItem;

        Boolean ready;

        SpriteFont menuFont;
        string selected;
        Vector2 createGamePos;
        string createGameText;
        Vector2 findGamePos;
        string findGameText;

        Vector2 returnToMainPos;
        string returnToMainText;

        KeyboardState previousKeyboardState;
        KeyboardState currentKeyboardState;

        GamerCollection<NetworkGamer> players;
        AvailableNetworkSessionCollection availableSessions;

        int selectedSessionIndex;

        private enum lobbyState
        {
            Welcome,
            CreateGame,
            FindGame,
            Connected
        }

        lobbyState currentState;

        public lobbyGUI()
        {
            currentState = lobbyState.Welcome;
            selectedSessionIndex = 0;
            ready = false;
        }

        public void LoadContent(ContentManager contentManager, GraphicsDeviceManager graphics)
        {
            background = contentManager.Load<Texture2D>("background");
            spotLight = contentManager.Load<Texture2D>("spotlight");
            cs = contentManager.Load<Texture2D>("cs");
            selectGameScreen = contentManager.Load<Texture2D>("selectGameScreen");
            backgroundPos = new Vector2(0f, 0f);
            spotLightPos = new Vector2(100f, graphics.GraphicsDevice.Viewport.Height - 98);
            spotLightCenter = new Vector2(800f, 800f);
            spotLightVelocity = new Vector2(-100, 33);
            csPos = new Vector2(10f, graphics.GraphicsDevice.Viewport.Height - 98);

            zero = new Vector2(0, 0);

            MaxX = graphics.GraphicsDevice.Viewport.Width;
            MinX = 0;
            MaxY = graphics.GraphicsDevice.Viewport.Height;
            MinY = 100;

            scale = MaxX / 1600f;

            //playerlist stuff
            checkedBox = contentManager.Load<Texture2D>("checkedBox");
            deselectBox = contentManager.Load<Texture2D>("deselectBox");
            emptySelectBox = contentManager.Load<Texture2D>("emptySelectBox");
            menuItem = contentManager.Load<Texture2D>("menuItem");

            

            //menu fonts
            menuFont = contentManager.Load<SpriteFont>("menuFont");
            createGamePos = new Vector2(100f, MaxY / 3);
            createGameText = "Create Game";
            selected = createGameText;

            findGamePos = new Vector2(100f, (MaxY / 3) + 60);
            findGameText = "Find Game";

            returnToMainPos = new Vector2(MaxX / 2, MaxY - 120);
            returnToMainText = "press [ X ] to return to main menu";

            
        }

        public void UnloadContent()
        {
            
        }

        public long Update(GameTime gameTime, NetworkGame networkGame)
        {
            
            UpdateSpotLight(gameTime);
            currentKeyboardState = Keyboard.GetState();

            if (networkGame.sessionExists())
            {
                players = networkGame.LocalGamers;
            }

            //check inputs
            switch (currentState)
            {
                case lobbyState.Welcome:
                    if (selected == createGameText)
                    {
                        if (currentKeyboardState.IsKeyDown(Keys.Enter) && previousKeyboardState.IsKeyUp(Keys.Enter))
                            currentState = lobbyState.CreateGame;
                        if (currentKeyboardState.IsKeyDown(Keys.Down))
                            selected = findGameText;
                    }
                    else
                    {
                        if (currentKeyboardState.IsKeyDown(Keys.Enter) && previousKeyboardState.IsKeyUp(Keys.Enter))
                            currentState = lobbyState.FindGame;
                        if (currentKeyboardState.IsKeyDown(Keys.Up))
                            selected = createGameText;
                    }
                    break;

                case lobbyState.CreateGame:
                    if (currentKeyboardState.IsKeyDown(Keys.X) && previousKeyboardState.IsKeyUp(Keys.X))
                    {
                        currentState = lobbyState.Welcome;
                        ready = false;
                        if (networkGame.sessionExists())
                        {
                            players = null;
                            networkGame.LeaveSession();
                        }
                    }
                    if (currentKeyboardState.IsKeyDown(Keys.Y) && previousKeyboardState.IsKeyUp(Keys.Y))
                    {
                        currentState = lobbyState.Connected;
                        networkGame.CreateSession(4);
                    }   
                    break;

                case lobbyState.FindGame:
                    if (currentKeyboardState.IsKeyDown(Keys.X) && previousKeyboardState.IsKeyUp(Keys.X))
                    {
                        currentState = lobbyState.Welcome;
                        ready = false;
                    }
                    availableSessions = networkGame.FindSessions();
                    if (availableSessions != null)
                    {
                        networkGame.JoinSession(availableSessions[0]);
                        currentState = lobbyState.Connected;

                        availableSessions.Dispose();
                        availableSessions = null;
                    }
                    break;

                case lobbyState.Connected:
                    if (currentKeyboardState.IsKeyDown(Keys.X) && previousKeyboardState.IsKeyUp(Keys.X))
                    {
                        ready = false;
                        if (networkGame.sessionExists())
                        {
                            players = null;
                            networkGame.LeaveSession();
                        }
                        currentState = lobbyState.Welcome;
                    }
                    if (currentKeyboardState.IsKeyDown(Keys.R) && previousKeyboardState.IsKeyUp(Keys.R))
                        ready = true;

                    if (networkGame.sessionExists())
                    {
                        players = networkGame.LocalGamers;
                    }
                    else
                        currentState = lobbyState.Welcome;


                    break;

            }
            previousKeyboardState = Keyboard.GetState();

            return 1;
        }

        /// <summary>
        /// Draws the lobby GUI. Has different states for difference menu configurations
        /// </summary>
        public long Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(background, backgroundPos, null, Color.White, 0, zero, scale, SpriteEffects.None, 0);
            spriteBatch.Draw(cs, csPos, null, Color.White, 0, zero, 0.5f, SpriteEffects.None, 0);
            spriteBatch.Draw(spotLight, spotLightPos, null, Color.White, 0, spotLightCenter, 1f, SpriteEffects.None, 0);

            switch (currentState)
            {
                case lobbyState.Welcome:
                    spriteBatch.DrawString(menuFont, "press [ Home ] to login", new Vector2(350, 20), Color.LightGray, 0f, zero, .6f, SpriteEffects.None, 0.5f);
                    if (selected == createGameText)
                        spriteBatch.DrawString(menuFont, createGameText, createGamePos, Color.Red, 0, zero, 1f, SpriteEffects.None, 0.5f);
                    else
                        spriteBatch.DrawString(menuFont, createGameText, createGamePos, Color.Gray, 0, zero, 1f,SpriteEffects.None, 0.5f);
                    if (selected == findGameText)
                        spriteBatch.DrawString(menuFont, findGameText, findGamePos, Color.Red, 0, zero, 1f, SpriteEffects.None, 0.5f);
                    else
                        spriteBatch.DrawString(menuFont, findGameText, findGamePos, Color.Gray, 0, zero, 1f, SpriteEffects.None, 0.5f);
                    break;
                
                case lobbyState.CreateGame:
                    DrawPlayerList(spriteBatch);
                    spriteBatch.DrawString(menuFont, "You are now the Host!", new Vector2(MaxX / 2, MaxY / 3), Color.White, 0f, zero, .7f, SpriteEffects.None, 0.5f);
                    spriteBatch.DrawString(menuFont, "press: Y to continue", new Vector2(MaxX /2, (MaxY / 3) + menuFont.LineSpacing), Color.White, 0f, zero, .6f, SpriteEffects.None, 0.5f);
                    spriteBatch.DrawString(menuFont, "X to return to main menu", new Vector2(MaxX /2 + 40, (MaxY / 3) + 2 * menuFont.LineSpacing), Color.White, 0f, zero, .6f, SpriteEffects.None, 0.5f);

                    break;

                case lobbyState.FindGame:
                    spriteBatch.Draw(selectGameScreen, backgroundPos, null, Color.White, 0, zero, scale, SpriteEffects.None, 0);
                    if(availableSessions == null)
                        spriteBatch.DrawString(menuFont, "searching for available games ....", new Vector2(150, 100), Color.Gray, 0f, zero, .7f, SpriteEffects.None, 0.5f);
                    else if (availableSessions.Count == 0)
                        spriteBatch.DrawString(menuFont, "no games currently available, searching ....", new Vector2(150, 100), Color.Gray, 0f, zero, .7f, SpriteEffects.None, 0.5f);
                    else
                    {
                        for (int sessionIndex = 0; sessionIndex < availableSessions.Count; sessionIndex++)
                        {
                            Color color = Color.Gray;

                            if (sessionIndex == selectedSessionIndex)
                                color = Color.Red;

                            spriteBatch.DrawString(menuFont, availableSessions[sessionIndex].HostGamertag, new Vector2(150, 100 + sessionIndex * menuFont.LineSpacing), color, 0f, zero, .7f, SpriteEffects.None, 0.5f);
                            spriteBatch.DrawString(menuFont, availableSessions[sessionIndex].CurrentGamerCount + " / " + availableSessions[sessionIndex].OpenPublicGamerSlots + availableSessions[sessionIndex].OpenPrivateGamerSlots,
                                                   new Vector2(400, 100 + sessionIndex * menuFont.LineSpacing), color, 0f, zero, .7f, SpriteEffects.None, 0.5f);
                        }
                    }
                    break;

                case lobbyState.Connected:
                    DrawPlayerList(spriteBatch);
                    spriteBatch.DrawString(menuFont, "Waiting for ready players...", new Vector2(MaxX / 2, MaxY / 2), Color.White, 0f, zero, .7f, SpriteEffects.None, 0.5f);
                    break;
            }

            return 1;
        }

        private void UpdateSpotLight(GameTime gameTime)
        {
            spotLightPos = new Vector2(spotLightPos.X + spotLightVelocity.X * (float)gameTime.ElapsedGameTime.TotalSeconds,
                                        spotLightPos.Y + spotLightVelocity.Y * (float)gameTime.ElapsedGameTime.TotalSeconds);

            if (spotLightPos.X > MaxX || spotLightPos.X < MinX) //right or left wall
            {
                spotLightVelocity = new Vector2(spotLightVelocity.X * -1, spotLightVelocity.Y);
            }
            else if (spotLightPos.Y > MaxY || spotLightPos.Y < MinY) //top or bottom wall
            {
                spotLightVelocity = new Vector2(spotLightVelocity.X, spotLightVelocity.Y * -1);
            }
        }


        private void DrawPlayerList(SpriteBatch spriteBatch)
        {
            
            //reference point
            Vector2 topOfList = new Vector2(MaxX / 8, MaxY / 4);

            //command info
            spriteBatch.DrawString(menuFont, "Current Players", new Vector2(topOfList.X + 15, topOfList.Y - 25), Color.White);
            spriteBatch.DrawString(menuFont, "Command Options:          to mark your self as ready", new Vector2(20, 20), Color.Gray, 0f, zero, .6f, SpriteEffects.None, 1f);
            spriteBatch.DrawString(menuFont, "type players # to toggle chat", new Vector2(175, 40), Color.Gray, 0f, zero, .6f, SpriteEffects.None, 1f);
            spriteBatch.DrawString(menuFont, "return to main menu", new Vector2(175, 60), Color.Gray, 0f, zero, .6f, SpriteEffects.None, 1f);
            spriteBatch.DrawString(menuFont, "R", new Vector2(145, 20), Color.DarkGreen, 0f, zero, .6f, SpriteEffects.None, 1f);
            spriteBatch.DrawString(menuFont, "#", new Vector2(145, 40), Color.DarkGreen, 0f, zero, .6f, SpriteEffects.None, 1f);
            spriteBatch.DrawString(menuFont, "X", new Vector2(145, 60), Color.DarkGreen, 0f, zero, .6f, SpriteEffects.None, 1f);
            
            
            spriteBatch.Draw(menuItem, topOfList, null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);

            //player 1
            spriteBatch.DrawString(menuFont, "Ready", new Vector2(topOfList.X + 5, topOfList.Y + 45), Color.DarkGray, 0f, zero, .6f, SpriteEffects.None, 1f);
            if(!ready)
                spriteBatch.Draw(emptySelectBox, new Vector2(topOfList.X - 32, topOfList.Y + 35), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            else
                spriteBatch.Draw(checkedBox, new Vector2(topOfList.X - 32, topOfList.Y + 30), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            
            spriteBatch.DrawString(menuFont, "Chat with", new Vector2(topOfList.X + 152, topOfList.Y + 45), Color.DarkGray, 0f, zero, .6f, SpriteEffects.None, 1f);
            Boolean chatwith = false;  // change to reflect info from network, move to update and create one for each player
            if (!chatwith)
                spriteBatch.Draw(deselectBox, new Vector2(topOfList.X +218, topOfList.Y + 35), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            else
                spriteBatch.Draw(checkedBox, new Vector2(topOfList.X + 218, topOfList.Y + 30), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);

            //player 2
            spriteBatch.Draw(menuItem, new Vector2(topOfList.X, topOfList.Y + 65), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            spriteBatch.DrawString(menuFont, "Ready", new Vector2(topOfList.X + 5, topOfList.Y + 110), Color.DarkGray, 0f, zero, .6f, SpriteEffects.None, 1f);
            if (!ready)
                spriteBatch.Draw(emptySelectBox, new Vector2(topOfList.X - 32, topOfList.Y + 100), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            else
                spriteBatch.Draw(checkedBox, new Vector2(topOfList.X - 32, topOfList.Y + 95), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);

            spriteBatch.DrawString(menuFont, "Chat with", new Vector2(topOfList.X + 152, topOfList.Y + 110), Color.DarkGray, 0f, zero, .6f, SpriteEffects.None, 1f);
            // change to reflect info from network
            if (!chatwith)
                spriteBatch.Draw(deselectBox, new Vector2(topOfList.X + 218, topOfList.Y + 100), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            else
                spriteBatch.Draw(checkedBox, new Vector2(topOfList.X + 218, topOfList.Y + 95), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            
            //player 3
            spriteBatch.Draw(menuItem, new Vector2(topOfList.X, topOfList.Y + 130), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            spriteBatch.DrawString(menuFont, "Ready", new Vector2(topOfList.X + 5, topOfList.Y + 175), Color.DarkGray, 0f, zero, .6f, SpriteEffects.None, 1f);
            if (!ready)
                spriteBatch.Draw(emptySelectBox, new Vector2(topOfList.X - 32, topOfList.Y + 165), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            else
                spriteBatch.Draw(checkedBox, new Vector2(topOfList.X - 32, topOfList.Y + 160), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);

            spriteBatch.DrawString(menuFont, "Chat with", new Vector2(topOfList.X + 152, topOfList.Y + 175), Color.DarkGray, 0f, zero, .6f, SpriteEffects.None, 1f);
            
            if (!chatwith)
                spriteBatch.Draw(deselectBox, new Vector2(topOfList.X + 218, topOfList.Y + 165), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            else
                spriteBatch.Draw(checkedBox, new Vector2(topOfList.X + 218, topOfList.Y + 160), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            
            //player 4
            spriteBatch.Draw(menuItem, new Vector2(topOfList.X, topOfList.Y + 195), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            spriteBatch.DrawString(menuFont, "Ready", new Vector2(topOfList.X + 5, topOfList.Y + 240), Color.DarkGray, 0f, zero, .6f, SpriteEffects.None, 1f);
            if (!ready)
                spriteBatch.Draw(emptySelectBox, new Vector2(topOfList.X - 32, topOfList.Y + 230), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            else
                spriteBatch.Draw(checkedBox, new Vector2(topOfList.X - 32, topOfList.Y + 225), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);

            spriteBatch.DrawString(menuFont, "Chat with", new Vector2(topOfList.X + 152, topOfList.Y + 240), Color.DarkGray, 0f, zero, .6f, SpriteEffects.None, 1f);
            if (!chatwith)
                spriteBatch.Draw(deselectBox, new Vector2(topOfList.X + 218, topOfList.Y + 230), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            else
                spriteBatch.Draw(checkedBox, new Vector2(topOfList.X + 218, topOfList.Y + 225), null, Color.White, 0, zero, scale, SpriteEffects.None, 1f);
            
            
            //draw player names
            if (players != null)
            {
                for (int g = 0; g < players.Count; g++)
                {
                    spriteBatch.DrawString(menuFont, players[g].Gamertag, new Vector2(topOfList.X + 10, topOfList.Y + 10 + 65*g), Color.White, 0f, zero, .75f, SpriteEffects.None, 1f);
                }
            }
        }
    }
}
